require "spec_helper"

describe "condition" do
  context "application" do
    before :all do
      class Person
        include Rulz
        attr_accessor :name, :age
      
        def initialize(name, age)
          @name = name
          @age = age
        end
      
        define_rulz do
          condition "older than" do |age|
            its.age > age
          end
        
          condition "younger than" do |age|
            opposite_of "older than", age
          end
        
          condition "middle aged" do
            condition("older than", 40) and condition("younger than", 50)
          end
        
        end      
      end
    end
  
    it "recognizes when conditions are true" do
      billy = Person.new("Billy", 54)
      billy.condition_true?("older than", 50).should be_true
      bobby = Person.new("Bobby", 23)
      bobby.condition_true?("older than", 50).should be_false
    end
  
    it "recognizes when conditions are false" do
      bilbo = Person.new("Bilbo", 46)
      bilbo.condition_false?("older than", 23).should be_false
      baggins = Person.new("Baggins", 12)
      baggins.condition_false?("older than", 23).should be_true    
    end
  
    it "can define conditions based on other conditions" do
      bobby = Person.new("Bobby", 11)
      bobby.condition_true?("younger than", 13).should be_true
      billy = Person.new("Billy", 45)
      billy.condition_true?("middle aged").should be_true
    end
  end
  
  context "come predefined in modules" do
    before :all do
      class String
        include Rulz
        include Rulz::Conditions::String
        include Rulz::Conditions::Comparison
      end
      class TrueClass
        include Rulz
        include Rulz::Conditions::Boolean
      end
      class FalseClass
        include Rulz
        include Rulz::Conditions::Boolean
      end
      class Fixnum
        include Rulz
        include Rulz::Conditions::Number
      end
    end
    
    it "should recognize contains" do
      "foobar".condition_true?("contains", "bar").should be_true
      "foobar".condition_true?("contains", "baz").should be_false     
    end
    
    it "should recognize matches" do
      "123".condition_true?("matches", /[0-4][0-4][0-4]/).should be_true
      "baz".condition_true?("matches",  /[0-4][0-4][0-4]/).should be_false
    end
    
    it "should recognize does not match" do
      "baz".condition_true?("does not match",  /[0-4][0-4][0-4]/).should be_true 
      "123".condition_true?("does not match", /[0-4][0-4][0-4]/).should be_false           
    end
    
    it "should recognize like" do
      "abc".condition_true?("like", "ABC").should be_true
      "123".condition_true?("like", "ABC").should be_false     
    end
    
    it "should recognize not like" do
      "123".condition_true?("not like", "ABC").should be_true      
      "abc".condition_true?("not like", "ABC").should be_false
    end
        
    it "should recognize equal to" do
      "abc".condition_true?("equal to", "abc").should be_true
      "ABC".condition_true?("equal to", "abc").should be_false
    end
    
    it "should recognize not equal to" do
      "ABC".condition_true?("not equal to", "abc").should be_true
      "abc".condition_true?("not equal to", "abc").should be_false
    end
    
    it "should recognize greater than" do
      "b".condition_true?("greater than", "a").should be_true
      "a".condition_true?("greater than", "b").should be_false
    end

    it "should recognize less than" do
      "a".condition_true?("less than", "b").should be_true
      "b".condition_true?("less than", "a").should be_false
    end
    
    it "should recognize true" do
      true.condition_true?("is true").should be_true
      false.condition_true?("is true").should be_false
    end
    
    it "should recognize false" do
      false.condition_true?("is false").should be_true
      true.condition_true?("is false").should be_false      
    end
    
    it "should recognize is zero" do
      0.condition_true?("is zero").should be_true
      1.condition_true?("is zero").should be_false
    end
    
    it "should recognize is positive" do
      1.condition_true?("is positive").should be_true        
      0.condition_true?("is positive").should be_false
    end

    it "should recognize is negative" do
      (-1).condition_true?("is negative").should be_true
      1.condition_true?("is negative").should be_false
    end

    
  end  
  
end